# This file was auto-generated by Fern from our API Definition.

import typing
from .environment import PinnacleEnvironment
import httpx
from .core.client_wrapper import SyncClientWrapper
from .send.client import SendClient
from .core.request_options import RequestOptions
from .types.rcs_functionalities import RcsFunctionalities
from .core.pydantic_utilities import parse_obj_as
from .errors.bad_request_error import BadRequestError
from .errors.unauthorized_error import UnauthorizedError
from .errors.internal_server_error import InternalServerError
from json.decoder import JSONDecodeError
from .core.api_error import ApiError
from .types.company import Company
from .types.company_details import CompanyDetails
from .types.company_contact import CompanyContact
from .types.point_of_contact import PointOfContact
from .types.optionals import Optionals
from .types.register_company_response import RegisterCompanyResponse
from .core.serialization import convert_and_respect_annotation_metadata
from .types.update_company_response import UpdateCompanyResponse
from .core.client_wrapper import AsyncClientWrapper
from .send.client import AsyncSendClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class Pinnacle:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : PinnacleEnvironment
        The environment to use for requests from the client. from .environment import PinnacleEnvironment



        Defaults to PinnacleEnvironment.DEFAULT



    api_key : str
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from rcs import Pinnacle

    client = Pinnacle(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: PinnacleEnvironment = PinnacleEnvironment.DEFAULT,
        api_key: str,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = timeout if timeout is not None else 60 if httpx_client is None else None
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.send = SendClient(client_wrapper=self._client_wrapper)

    def get_rcs_functionality(
        self, *, phone_number: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> RcsFunctionalities:
        """
        Retrieve the RCS functionality of a phone number. For example checks if a phone number can receive RCS message and if it can receive RCS carousels.

        Parameters
        ----------
        phone_number : typing.Optional[str]
            The phone number to check for RCS functionality. Should be in E.164 format (i.e. +12345678901).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RcsFunctionalities
            Successfully retrieved RCS functionality

        Examples
        --------
        from rcs import Pinnacle

        client = Pinnacle(
            api_key="YOUR_API_KEY",
        )
        client.get_rcs_functionality()
        """
        _response = self._client_wrapper.httpx_client.request(
            "rcs_functionality",
            method="GET",
            params={
                "phoneNumber": phone_number,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    RcsFunctionalities,
                    parse_obj_as(
                        type_=RcsFunctionalities,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_company(
        self,
        *,
        company_id: typing.Optional[int] = None,
        company_name: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Company]:
        """
        Retrieve the company's information (i.e. approval status, company name, etc.). Search by company ID or company name.

        Parameters
        ----------
        company_id : typing.Optional[int]
            The unique identifier for the company

        company_name : typing.Optional[str]
            The name of the company

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Company]
            Successfully retrieved company information

        Examples
        --------
        from rcs import Pinnacle

        client = Pinnacle(
            api_key="YOUR_API_KEY",
        )
        client.get_company()
        """
        _response = self._client_wrapper.httpx_client.request(
            "company",
            method="GET",
            params={
                "companyId": company_id,
                "companyName": company_name,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Company],
                    parse_obj_as(
                        type_=typing.List[Company],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def register_company(
        self,
        *,
        company: CompanyDetails,
        company_contact: CompanyContact,
        point_of_contact: PointOfContact,
        optionals: typing.Optional[Optionals] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RegisterCompanyResponse:
        """
        Register a company for RCS with the Pinnacle platform

        Parameters
        ----------
        company : CompanyDetails

        company_contact : CompanyContact

        point_of_contact : PointOfContact

        optionals : typing.Optional[Optionals]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RegisterCompanyResponse
            Successfully registered/updated brand

        Examples
        --------
        from rcs import CompanyContact, CompanyDetails, Pinnacle, PointOfContact

        client = Pinnacle(
            api_key="YOUR_API_KEY",
        )
        client.register_company(
            company=CompanyDetails(
                name="name",
                address="address",
                ein="ein",
                description="description",
                brand_color="brandColor",
                logo_url="logoUrl",
                hero_url="heroUrl",
            ),
            company_contact=CompanyContact(
                primary_website_url="primaryWebsiteUrl",
                primary_website_label="primaryWebsiteLabel",
                primary_phone="primaryPhone",
                primary_phone_label="primaryPhoneLabel",
                primary_email="primaryEmail",
                primary_email_label="primaryEmailLabel",
                privacy_policy_url="privacyPolicyUrl",
                tos_url="tosUrl",
            ),
            point_of_contact=PointOfContact(
                poc_name="pocName",
                poc_title="pocTitle",
                poc_email="pocEmail",
            ),
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "company/register",
            method="POST",
            json={
                "company": convert_and_respect_annotation_metadata(
                    object_=company, annotation=CompanyDetails, direction="write"
                ),
                "companyContact": convert_and_respect_annotation_metadata(
                    object_=company_contact, annotation=CompanyContact, direction="write"
                ),
                "pointOfContact": convert_and_respect_annotation_metadata(
                    object_=point_of_contact, annotation=PointOfContact, direction="write"
                ),
                "optionals": convert_and_respect_annotation_metadata(
                    object_=optionals, annotation=Optionals, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    RegisterCompanyResponse,
                    parse_obj_as(
                        type_=RegisterCompanyResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update_company(
        self,
        *,
        company_id: str,
        company: typing.Optional[Company] = OMIT,
        company_contact: typing.Optional[CompanyContact] = OMIT,
        point_of_contact: typing.Optional[PointOfContact] = OMIT,
        optionals: typing.Optional[Optionals] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateCompanyResponse:
        """
        Update a company on the Pinnacle platform

        Parameters
        ----------
        company_id : str

        company : typing.Optional[Company]

        company_contact : typing.Optional[CompanyContact]

        point_of_contact : typing.Optional[PointOfContact]

        optionals : typing.Optional[Optionals]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateCompanyResponse
            Successfully registered/updated brand

        Examples
        --------
        from rcs import Pinnacle

        client = Pinnacle(
            api_key="YOUR_API_KEY",
        )
        client.update_company(
            company_id="companyId",
        )
        """
        _response = self._client_wrapper.httpx_client.request(
            "company/update",
            method="POST",
            json={
                "companyId": company_id,
                "company": convert_and_respect_annotation_metadata(
                    object_=company, annotation=Company, direction="write"
                ),
                "companyContact": convert_and_respect_annotation_metadata(
                    object_=company_contact, annotation=CompanyContact, direction="write"
                ),
                "pointOfContact": convert_and_respect_annotation_metadata(
                    object_=point_of_contact, annotation=PointOfContact, direction="write"
                ),
                "optionals": convert_and_respect_annotation_metadata(
                    object_=optionals, annotation=Optionals, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UpdateCompanyResponse,
                    parse_obj_as(
                        type_=UpdateCompanyResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncPinnacle:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : PinnacleEnvironment
        The environment to use for requests from the client. from .environment import PinnacleEnvironment



        Defaults to PinnacleEnvironment.DEFAULT



    api_key : str
    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from rcs import AsyncPinnacle

    client = AsyncPinnacle(
        api_key="YOUR_API_KEY",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: PinnacleEnvironment = PinnacleEnvironment.DEFAULT,
        api_key: str,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = timeout if timeout is not None else 60 if httpx_client is None else None
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            api_key=api_key,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.send = AsyncSendClient(client_wrapper=self._client_wrapper)

    async def get_rcs_functionality(
        self, *, phone_number: typing.Optional[str] = None, request_options: typing.Optional[RequestOptions] = None
    ) -> RcsFunctionalities:
        """
        Retrieve the RCS functionality of a phone number. For example checks if a phone number can receive RCS message and if it can receive RCS carousels.

        Parameters
        ----------
        phone_number : typing.Optional[str]
            The phone number to check for RCS functionality. Should be in E.164 format (i.e. +12345678901).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RcsFunctionalities
            Successfully retrieved RCS functionality

        Examples
        --------
        import asyncio

        from rcs import AsyncPinnacle

        client = AsyncPinnacle(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.get_rcs_functionality()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "rcs_functionality",
            method="GET",
            params={
                "phoneNumber": phone_number,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    RcsFunctionalities,
                    parse_obj_as(
                        type_=RcsFunctionalities,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_company(
        self,
        *,
        company_id: typing.Optional[int] = None,
        company_name: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[Company]:
        """
        Retrieve the company's information (i.e. approval status, company name, etc.). Search by company ID or company name.

        Parameters
        ----------
        company_id : typing.Optional[int]
            The unique identifier for the company

        company_name : typing.Optional[str]
            The name of the company

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[Company]
            Successfully retrieved company information

        Examples
        --------
        import asyncio

        from rcs import AsyncPinnacle

        client = AsyncPinnacle(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.get_company()


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "company",
            method="GET",
            params={
                "companyId": company_id,
                "companyName": company_name,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    typing.List[Company],
                    parse_obj_as(
                        type_=typing.List[Company],  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def register_company(
        self,
        *,
        company: CompanyDetails,
        company_contact: CompanyContact,
        point_of_contact: PointOfContact,
        optionals: typing.Optional[Optionals] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> RegisterCompanyResponse:
        """
        Register a company for RCS with the Pinnacle platform

        Parameters
        ----------
        company : CompanyDetails

        company_contact : CompanyContact

        point_of_contact : PointOfContact

        optionals : typing.Optional[Optionals]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RegisterCompanyResponse
            Successfully registered/updated brand

        Examples
        --------
        import asyncio

        from rcs import AsyncPinnacle, CompanyContact, CompanyDetails, PointOfContact

        client = AsyncPinnacle(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.register_company(
                company=CompanyDetails(
                    name="name",
                    address="address",
                    ein="ein",
                    description="description",
                    brand_color="brandColor",
                    logo_url="logoUrl",
                    hero_url="heroUrl",
                ),
                company_contact=CompanyContact(
                    primary_website_url="primaryWebsiteUrl",
                    primary_website_label="primaryWebsiteLabel",
                    primary_phone="primaryPhone",
                    primary_phone_label="primaryPhoneLabel",
                    primary_email="primaryEmail",
                    primary_email_label="primaryEmailLabel",
                    privacy_policy_url="privacyPolicyUrl",
                    tos_url="tosUrl",
                ),
                point_of_contact=PointOfContact(
                    poc_name="pocName",
                    poc_title="pocTitle",
                    poc_email="pocEmail",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "company/register",
            method="POST",
            json={
                "company": convert_and_respect_annotation_metadata(
                    object_=company, annotation=CompanyDetails, direction="write"
                ),
                "companyContact": convert_and_respect_annotation_metadata(
                    object_=company_contact, annotation=CompanyContact, direction="write"
                ),
                "pointOfContact": convert_and_respect_annotation_metadata(
                    object_=point_of_contact, annotation=PointOfContact, direction="write"
                ),
                "optionals": convert_and_respect_annotation_metadata(
                    object_=optionals, annotation=Optionals, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    RegisterCompanyResponse,
                    parse_obj_as(
                        type_=RegisterCompanyResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update_company(
        self,
        *,
        company_id: str,
        company: typing.Optional[Company] = OMIT,
        company_contact: typing.Optional[CompanyContact] = OMIT,
        point_of_contact: typing.Optional[PointOfContact] = OMIT,
        optionals: typing.Optional[Optionals] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateCompanyResponse:
        """
        Update a company on the Pinnacle platform

        Parameters
        ----------
        company_id : str

        company : typing.Optional[Company]

        company_contact : typing.Optional[CompanyContact]

        point_of_contact : typing.Optional[PointOfContact]

        optionals : typing.Optional[Optionals]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateCompanyResponse
            Successfully registered/updated brand

        Examples
        --------
        import asyncio

        from rcs import AsyncPinnacle

        client = AsyncPinnacle(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.update_company(
                company_id="companyId",
            )


        asyncio.run(main())
        """
        _response = await self._client_wrapper.httpx_client.request(
            "company/update",
            method="POST",
            json={
                "companyId": company_id,
                "company": convert_and_respect_annotation_metadata(
                    object_=company, annotation=Company, direction="write"
                ),
                "companyContact": convert_and_respect_annotation_metadata(
                    object_=company_contact, annotation=CompanyContact, direction="write"
                ),
                "pointOfContact": convert_and_respect_annotation_metadata(
                    object_=point_of_contact, annotation=PointOfContact, direction="write"
                ),
                "optionals": convert_and_respect_annotation_metadata(
                    object_=optionals, annotation=Optionals, direction="write"
                ),
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                return typing.cast(
                    UpdateCompanyResponse,
                    parse_obj_as(
                        type_=UpdateCompanyResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
            if _response.status_code == 400:
                raise BadRequestError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    )
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        raise ApiError(status_code=_response.status_code, body=_response_json)


def _get_base_url(*, base_url: typing.Optional[str] = None, environment: PinnacleEnvironment) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception("Please pass in either base_url or environment to construct the client")
